#!/usr/bin/perl -w
#############################################################################
## Name:        wxactivex_template.pl
## Purpose:     Template Builder for ActiveX Control
## Author:      Mark Dootson
## Created:     2008-04-7
## SVN-ID:      $Id: wxactivex_template.pl 2367 2008-04-12 14:05:11Z mdootson $
## Copyright:   (c) 2008 Mark Dootson
## Licence:     This program is free software; you can redistribute it and/or
##              modify it under the same terms as Perl itself
#############################################################################

use strict;
use Wx;
use Wx::ActiveX::Template;

run_wxactivex_template();

1;



